<?php

abstract class BaseListScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const PAGE_SIZE = 9;

    const MOV_ICON_W = 52;
    const MOV_ICON_H = 70;
    const MOV_ICON_DX = 13;

    const OTHER_ICON_W = 48;
    const OTHER_ICON_H = 48;
    const OTHER_ICON_DX = 15;

    protected $debug = false;

    protected function __construct($id)
    {
        parent::__construct($id);

        $this->debug = is_file("/config/omov_debug");

        $this->set_folder_view_index_attr_name('view_index');

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_view_index(&$plugin_cookies) {
        return HD::get($plugin_cookies, 'view_index', 0);
    }

    protected function do_get_folder_views(&$plugin_cookies)
    {
        $loading_url = "plugin_file://%shell_ext%/icons/movies_small.png";
        $load_failed_url = "plugin_file://%shell_ext%/icons/no_movie_poster.png";

        $height = 700;

        $view = array(
            PluginRegularFolderView::view_params => array_merge(
                array(
                    ViewParams::num_cols => 1,
                    ViewParams::num_rows => 11,
                    ViewParams::paint_details => false,

                    ViewParams::scroll_animation_enabled => true,

                    ViewParams::content_box_y => 250,
                    ViewParams::content_box_height => $height,
                )),

            PluginRegularFolderView::base_view_item_params => array(
                ViewItemParams::item_padding_top => 0,
                ViewItemParams::item_padding_bottom => 0,
                ViewItemParams::icon_valign => VALIGN_CENTER,
                ViewItemParams::item_layout => HALIGN_LEFT,
                ViewItemParams::item_caption_width => 1570,
                ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                ViewItemParams::icon_dx => self::OTHER_ICON_DX,
                ViewItemParams::icon_width => 38,
                ViewItemParams::icon_height => 38,
            ),

            PluginRegularFolderView::not_loaded_view_item_params => array
            (
                ViewItemParams::icon_path => $loading_url,
            ),

            PluginRegularFolderView::load_failed_view_item_params => array
            (
                ViewItemParams::icon_path => $load_failed_url,
                ViewItemParams::icon_width => self::MOV_ICON_W,
                ViewItemParams::icon_height => self::MOV_ICON_H,
                ViewItemParams::icon_dx => self::MOV_ICON_DX,
            ),
        );
        return array($view, $view);
    }

    public function custom_action($control_id,
        $params=null, $timeout_ms=null, $show_dialog_delay_ms=null,
        $skip_on_play_restart=false)
    {
        $a = UserInputHandlerRegistry::create_action($this,
            $control_id, $params, $timeout_ms, $show_dialog_delay_ms);
        if ($skip_on_play_restart)
            $a[GuiAction::flags] = GUI_ACTION_FLAG_SKIP_ON_PLAY_RESTART;
        return $a;
    }

}

?>
