<?php

class DlState
{
    const GSTAT_PATH = '/tmp/run/dl_gstat.txt';
    const BASE_DIR_PATH = '/flashdata/dl_cache';
    const BASE_TMP_DIR_PATH = '/tmp/run/dl_cache';

    public $gstat = array();
    public $gstat_mtime = -2;

    public $cur_g;
    public $all_groups;
    public $all_groups_loaded = false;

    public function __construct() {
        $this->cur_g = $this->st_initialilzer();
        $this->all_groups = array();
    }

    private function st_initialilzer() {
        return (object) array(
            'group_id' => null, 'mtime' => -2, 'f_by_path' => null);
    }

    public function reload_gstat()
    {
        $path = self::GSTAT_PATH;
        clearstatcache(true, $path);

        if (!is_file($path)) {
            $this->gstat_mtime = -1;
            $changed = (bool) $this->gstat;
            $this->gstat = array();
            return $changed;
        }

        $mtime = filemtime($path);
        if ($mtime === false)
            $mtime = -1;

        if ($mtime == $this->gstat_mtime)
            return false;

        $this->gstat_mtime = $mtime;
        $this->gstat = array();

        $lines = file($path);
        foreach ($lines as $line)
        {
            $line = rtrim($line);
            list ($st, $progress, $id) = explode(' ', $line, 3);
            if ($st == 'D') {
                unset($this->gstat[$id]);
            } else {
                $this->gstat[$id] = (object) array(
                    'status' => $st, 'progress' => $progress);
            }
        }

        return true;
    }

    private function get_torrent_state_path($group_id, $is_tmp)
    {
        $dir = $is_tmp ? self::BASE_TMP_DIR_PATH : self::BASE_DIR_PATH;
        return "$dir/$group_id.json";
    }

    public function sync_state($group_id)
    {
        if ($group_id != $this->cur_g->group_id)
            $this->reload_state($group_id);
    }

    private function load_state_from_path($group_id, $path, &$g)
    {
        if (!is_file($path)) {
            $changed = $group_id != $g->group_id || $g->f_by_path;
            $g->mtime = -1;
            $g->f_by_path = null;
            $g->group_id = $group_id;
            return $changed;
        }


        $mtime = filemtime($path);
        if ($mtime === false)
            $mtime = -1;

        if ($group_id == $g->group_id &&
            $mtime == $g->mtime)
        {
            return false;
        }

        $g->group_id = $group_id;
        $g->name = '?';
        $g->mtime = $mtime;
        $g->f_by_path = null;
        $state = json_decode(file_get_contents($path));
        if ($state) {
            $g->name = $state->name;
            foreach ($state->files as $f)
                $g->f_by_path[$f->path] = $f;
        }
        return true;
    }

    public function reload_state($group_id)
    {
        clearstatcache();
        $path = $this->get_torrent_state_path($group_id, true);
        if (!is_file($path))
            $path = $this->get_torrent_state_path($group_id, false);

        return $this->load_state_from_path($group_id, $path, $this->cur_g);
    }

    ///////////////////////////////////////////////////////////////////////

    public function all_state_reset() {
        $this->all_groups = array();
        $this->all_groups_loaded = false;
    }

    public function all_state_reload() {
        $this->all_state_reset();
        $this->all_state_load();
    }

    public function all_state_ensure_loaded() {
        if (!$this->all_groups_loaded)
            $this->all_state_load();
    }

    public function all_state_load()
    {
        $dirs = array(self::BASE_TMP_DIR_PATH, self::BASE_DIR_PATH);

        $changed = false;
        $gidset = array();
        foreach ($dirs as $dir)
        {
            $fnames = scandir($dir);
            foreach ($fnames as $fname)
            {
                if ($fname[0] == '.')
                    continue;

                $len = strlen($fname);
                if ($len < 6 || substr($fname, $len - 5) != '.json')
                    continue;

                $group_id = substr($fname, 0, $len - 5);

                if (isset($gidset[$group_id]))
                    continue;
                $gidset[$group_id] = 1;

                $path = "$dir/$fname";

                $group_is_new = false;
                if (!isset($this->all_groups[$group_id]))
                {
                    hd_print("I: group appeared: $group_id");
                    $this->all_groups[$group_id] = $this->st_initialilzer();
                    $group_is_new = true;
                }

                $res = $this->load_state_from_path(
                    $group_id, $path, $this->all_groups[$group_id]);
                if ($res)
                {
                    if (!$group_is_new)
                        hd_print("I: group changed: $group_id");
                    $changed = true;
                }
            }
        }

        foreach ($this->all_groups as $group_id => $g)
        {
            if (isset($gidset[$group_id]))
                continue;

            hd_print("I: group disappeared: $group_id");
            unset($this->all_groups[$group_id]);
            $changed = true;
        }

        if ($changed)
        {
            uasort($this->all_groups, 'DlState::cmp_groups');

            hd_print("D: groups after reload: ".count($this->all_groups));
        }

        return $changed;
    }

    public static function cmp_groups($g1, $g2)
    {
        // TODO
        return strcmp($g1->group_id, $g2->group_id);
    }

    ///////////////////////////////////////////////////////////////////////

    public static function add_download_file(&$files,
        $path, $url, $size_mb)
    {
        $files[] = array(
            'path' => $path, 'url' => $url, 'size_mb' => $size_mb);
    }

    public static function add_download(
        $group_id, $group_name, $files, $params=null)
    {
        $id = md5($group_id);
        $id = (string) microtime(true) . "." . $id;
        $input_path = "/tmp/run/dl_input/$id";
        hd_print("Download: input_path $input_path");

        $in = array(
            'cmd_type' => 'add',
            'id' => $group_id,
            'name' => $group_name,
            'files' => $files);
        if ($params)
            $in['params'] = $params;

        file_put_contents($input_path, json_encode($in));
    }

    public static function remove_download($group_id, $file_paths=null)
    {
        $id = md5($group_id);
        $id = (string) microtime(true) . "." . $id;
        $input_path = "/tmp/run/dl_input/$id";

        $in = array(
            'cmd_type' => 'remove',
            'group_id' => $group_id);
        if ($file_paths)
            $in['file_paths'] = $file_paths;

        file_put_contents($input_path, json_encode($in));
    }
}

?>
