<?php

class DownloadsScreen extends BaseListScreen
{
    const ID = 'downloads';

    public static function get_media_url_str() {
        return self::ID;
    }

    ///////////////////////////////////////////////////////////////////////

    private $fast_timer_countdown = 0;

    private $dl;

    public function __construct() {
        parent::__construct(self::ID);
        $this->dl = MainHelper::$dl_state;
    }

    public function get_handler_id() {
        return self::ID;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        return parent::get_folder_view($media_url, $plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),
            GUI_EVENT_KEY_INFO => $this->custom_action('info'),
            GUI_EVENT_KEY_POPUP_MENU => $this->custom_action('popup'),
            GUI_EVENT_TIMER => $this->custom_action('timer'),
        );
    }

    public function get_timer($media_url)
    {
        $st = MainHelper::$state;
        if ($this->fast_timer_countdown > 0 || $st->installing_ts)
            return ActionFactory::timer(1000);
        return ActionFactory::timer(5000);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(
        MediaURL $media_url, &$plugin_cookies)
    {
        $this->dl->reload_gstat();
        $this->dl->all_state_ensure_loaded();

        $items = array();

        $th = MainHelper::$torrent_history;
        $mts = MainHelper::$state->nav_mts;

        foreach ($this->dl->all_groups as $group_id => $g)
        {
            $item_murl = MediaURL::encode(
                array('group_id' => $group_id));

            $caption = $g->name;

            $gstat = HD::get($this->dl->gstat, $group_id);
            $status = $gstat ? HD::get($gstat, 'status', 'N') : '';
            if ($status == 'L')
                $this->set_fast_timer();

            if ($status == 'O') {
                $url = 'plugin_file://icons/dim_dl_small.png';
            } else if (in_array($status, array('L', 'N'))) {
                $url = 'plugin_file://icons/dl_small.png';
            } else if ($status == 'E') {
                $url = 'plugin_file://icons/err_dl_small.png';
            } else {
                //$url = 'plugin_file://%shell_ext%/icons/movies_small.png';
                $url = 'plugin_file://icons/dim_dl_small.png';
            }

            $vip = array(ViewItemParams::icon_path => $url);
            $vip[ViewItemParams::item_caption_dy] = -2;
            $vip[ViewItemParams::item_caption_sel_dy] = -2;

            $eitems = array();

//            $tag = $th->make_torrent_tag($t->uid);
//            $witem = HD::get($th->witem_by_tag, $tag);
//            TorrentHistory::update_vip_eitems_by_witem($eitems, $witem);

            if ($gstat)
            {
                if ($eitems)
                {
                    $gap = array_shift($eitems);
                    if ($gap['type'] == 'gap')
                        $gap['width'] = 10;
                    array_unshift($eitems, $gap);
                }

                // XXX TODO: copy-n-paste?
                if ($status == 'O') {
                    $arr = array('type' => 'icon',
                        'url' => 'plugin_file://%shell_ext%/icons/watched.png',
                        'width' => 41, 'height' => 31,
                        'alt' => '100%');
                } else if ($status == 'E') {
                    $arr = array('type' => 'icon',
                        'url' => 'plugin_file://icons/err.png',
                        'dy' => -6,
                        'alt' => 'ERR');
                } else {
                    $text = "[$gstat->progress%]";
                    $color = $status == 'L' ? 34 : 8;
                    $arr = array('type' => 'text', 'color' => $color,
                        'size' => 'small', 'text' => "$text");
                }
                array_unshift($eitems, $arr);
                array_unshift($eitems, 
                    array('type' => 'gap', 'width' => 20));
            }

            if ($eitems) {
                $einf = array('items' => $eitems);
                $vip[ViewItemParams::item_extra_info] = json_encode($einf);
            }

            $items[] = array(
                PluginRegularFolderItem::media_url => $item_murl,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => $vip,
            );
        }

        return $items;
    }

    ///////////////////////////////////////////////////////////////////////

    private function show_info($g)
    {
        return null;
    }

    private function popup_add_dl_items(&$menu_items,
        $parent_murl, $group_id)
    {
        if (!is_dir("/tmp/run/dl_input"))
            return;
        if (!is_dir("/tmp/mnt/D/dune_downloads"))
            return;

        $this->dl->reload_state($group_id);
        $g = $this->dl->cur_g;

        if ($g->f_by_path)
        {
            foreach ($g->f_by_path as $path => $f)
            {
                $status = $f ? HD::get($f, 'status', 'N') : '';
                if ($status && $status != 'N')
                {
                    $menu_items[] = ActionFactory::get_menu_item(
                        T::t('popup_action_show_in_folder'),
                        MainHelper::$entry_handler->custom_action(
                            'show_in_folder',
                            array('group_id' => $group_id,
                                'file_path' => $path)));
                    break;
                }
            }

            $menu_items[] = ActionFactory::get_menu_item(
                T::g('navigator_popup_delete'),
                MainHelper::$entry_handler->custom_action(
                    'delete_torrent_data',
                    array('group_id' => $group_id)));
        }
        else
        {
            $menu_items[] = ActionFactory::get_menu_item(
                T::t('popup_action_download'),
                $this->custom_action('download'),
                'plugin_file://icons/dim_dl_small.png');
        }
    }

    private function popup($user_input, $parent_murl, $group_id)
    {
        $menu_items = array();
        $this->popup_add_dl_items($menu_items,
            $parent_murl, $group_id);
        if (!$menu_items)
            return null;
        return ActionFactory::show_popup_menu($menu_items);
    }

    public function set_fast_timer()
    {
        $this->fast_timer_countdown = 10;
    }

    public function speed_up_timer($parent_murl, &$plugin_cookies)
    {
        $this->set_fast_timer();
        return ActionFactory::change_behaviour(
            $this->get_action_map($parent_murl, $plugin_cookies),
            $this->get_timer($parent_murl));
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        if (HD::get($user_input, 'control_id') == 'timer') {
            hd_print('torrents D: timer');
        } else {
            hd_print('Torrents screen: handle_user_input:');
            foreach ($user_input as $key => $value)
                hd_print("  $key => $value");
        }

        $eh = MainHelper::$entry_handler;

        $murl_str = HD::get($user_input, 'chosen_media_url',
            HD::get($user_input, 'selected_media_url'));

        $murl = $murl_str ? json_decode($murl_str) : null;
        $group_id = HD::get($murl, 'group_id');
        $g = HD::get($this->dl->all_groups, $group_id);

        $parent_murl = MediaURL::decode($user_input->parent_media_url);
        $st = MainHelper::$state;
        $mts = $st->nav_mts;

        if ($user_input->control_id == 'info') {
            return $g ? $this->show_info($g) : null;
        }

        if ($user_input->control_id == 'popup')
            return $this->popup($user_input, $parent_murl, $g->group_id);

        if ($user_input->control_id == 'enter')
        {
            if (!$g)
                return null;

//            return ActionFactory::open_folder_ext(
//                TorrentsScreen::get_media_url_str(
//                    $parent_murl->movie_id,
//                    $parent_murl->season, $parent_murl->episode,
//                    false, $stack),
//                T::t('all_variants'), null, $sel_id, $post_action);
            return null;
        }

        if ($user_input->control_id == 'timer')
        {
            $changed1 = $this->dl->reload_gstat();
            $changed2 = $this->dl->all_state_load();
            $a = ActionFactory::change_behaviour(
                $this->get_action_map($parent_murl, $plugin_cookies),
                $this->get_timer($parent_murl));
            if ($this->fast_timer_countdown > 0)
                $this->fast_timer_countdown--;

            if ($changed1 || $changed2)
            {
                $a = ActionFactory::invalidate_folders(
                    array($user_input->parent_media_url), $a);
            }

            return $a;
        }

        return null;
    }
}
?>
