<?php

class TorrentFilesScreen extends BaseListScreen
{
    const ID = 'torrent_files';

    public static function get_media_url_str(
        $movie_id, $torrent_id, $stack)
    {
        $arr = array(
            'screen_id' => self::ID,
            'movie_id' => $movie_id,
            'torrent_id' => $torrent_id,
            'stack' => $stack,
        );
        return MediaURL::encode($arr);
    }

    private $fast_timer_countdown = 0;

    public function __construct() {
        parent::__construct(self::ID);
    }

    public function get_handler_id() {
        return self::ID;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        $this->set_folder_views(null);
        return parent::get_folder_view($media_url, $plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),
            GUI_EVENT_KEY_INFO => $this->custom_action('info'),
            GUI_EVENT_KEY_POPUP_MENU => $this->custom_action('popup'),
            GUI_EVENT_MENU_PLAYBACK_FINISH =>
                $this->custom_action("menu_playback_finish",
                    null, null, null, true),
            GUI_EVENT_TIMER => $this->custom_action('timer'),
        );
    }

    public function get_timer($media_url)
    {
        if ($this->fast_timer_countdown > 0)
            return ActionFactory::timer(1000);
        return ActionFactory::timer(5000);
    }

    ///////////////////////////////////////////////////////////////////////

    public static function get_item_media_url($id)
    {
        return json_encode(array('id' => $id));
    }

    private static function to_float_mb($v)
    {
        $len = strlen($v);
        if ($len <= 6)
            return $v <= 0 ? 0 : $v / 1048576.0;
        return (float) substr($v, 0, $len - 6) +
            substr($v, $len - 6) / 1048576.0;
    }
    
    // NOTE: here f->pos and f->len can exceed integer limits, so we have
    // to workaround.
    private static function make_progress($f)
    {
        $status = HD::get($f, 'status', 'N');
        if ($status == 'N')
            return 0;
        if ($status == 'O')
            return 100;

        if (!isset($f->pos) || !isset($f->len))
            return 0;

        $pos = self::to_float_mb($f->pos);
        $len = self::to_float_mb($f->len);
        if ($len <= 1 || $pos < 0)
            return 0;
        return min(100, (int) round(($pos / $len) * 100.0));
    }

    public function get_all_folder_items(
        MediaURL $media_url, &$plugin_cookies)
    {
        $items = array();

        $icon_url = 'plugin_file://%shell_ext%/icons/movies_small.png';

        $st = MainHelper::$state;
        $tfs = $st->nav_tfs;
        $th = MainHelper::$torrent_history;
        $mts = $st->nav_mts;
        $m = $mts->get_movie();
        $tor_series = $tfs->get_series();

        $dl = MainHelper::$dl_state;
        $dl->sync_state($tfs->get_group_id());

        foreach ($tor_series as $path => $ser)
        {
            $item_murl = TorrentFilesScreen::get_item_media_url($ser['id']);

            $caption = $ser['title'];
            if (isset($ser['size']))
                $caption .= ' (' . $ser['size'] . ')';

            $f = HD::get($dl->cur_g->f_by_path, $path);
            $status = $f ? HD::get($f, 'status', 'N') : '';
            if ($status == 'L')
                $this->set_fast_timer();

            if ($status == 'O')
                $url = 'plugin_file://icons/dim_dl_small.png';
            else if (in_array($status, array('L', 'N')))
                $url = 'plugin_file://icons/dl_small.png';
            else
                $url = $icon_url;

            $vip = array(ViewItemParams::icon_path => $url);

            $uid = $th->make_file_uid(
                $m->id, $tfs->get_torrent_uid(), $ser['id']);
            $witem = HD::get($th->witem_by_uid, $uid);
            $eitems = array();
            TorrentHistory::update_vip_eitems_by_witem($eitems, $witem);

            // XXX TODO: copy-n-paste?
            if ($f)
            {
                if ($eitems)
                {
                    $gap = array_shift($eitems);
                    if ($gap['type'] == 'gap')
                        $gap['width'] = 10;
                    array_unshift($eitems, $gap);
                }

                if ($status == 'O') {
                    $arr = array('type' => 'icon',
                        'url' => 'plugin_file://%shell_ext%/icons/watched.png',
                        'width' => 41, 'height' => 31,
                        'alt' => '100%');
                } else if ($status == 'E') {
                    $arr = array('type' => 'icon',
                        'url' => 'plugin_file://icons/err.png',
                        'dy' => -6,
                        'alt' => 'ERR');
                } else {
                    $progress = self::make_progress($f);
                    $text = "[$progress%]";
                    $color = $status == 'L' ? 34 : 8;
                    $arr = array('type' => 'text', 'color' => $color,
                        'size' => 'small', 'text' => "$text");
                }
                array_unshift($eitems, $arr);
                array_unshift($eitems, 
                    array('type' => 'gap', 'width' => 20));
            }

            if ($eitems) {
                $einf = array('items' => $eitems);
                $vip[ViewItemParams::item_extra_info] = json_encode($einf);
            }

            $items[] = array(
                PluginRegularFolderItem::media_url => $item_murl,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => $vip,
            );
        }

        return $items;
    }

    ///////////////////////////////////////////////////////////////////////

    private function enter($user_input)
    {
        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $autostart = $mts->get_autostart();

        $sel = json_decode($user_input->selected_media_url);
        $sel_id = $sel ? HD::get($sel, 'id') : '';

        $pos = HD::get($user_input, 'chosen_file_pos', -1);

        $st->play_torrent($st->nav_tfs, 0);
        $tfs = $st->play_tfs;

        $torrent_id = $tfs->get_torrent_id();
        $torrent_uid = $tfs->get_torrent_uid();
        $series = $tfs->get_series();

        $murl = MediaURL::decode($user_input->parent_media_url);
        $stack = $murl->stack . "TF:$sel_id;";

        MainHelper::$playback->starting_new_playback();

        $cfg = MainHelper::$config;
        $res = TorrentHelper::check_torrent_item_suitable_for_autoswitch_playback(
            $series, $sel_id);
        if ($res && $cfg->autoswitch_enabled()) {
            $e = HD::get($res, 'e', -1);
            $sel_id = $e >= 0 ? "E$e" : null;
            $vod_movie = TorrentHelper::get_all_episodes_vod_movie(
                $stack, $autostart, $torrent_id, $torrent_uid,
                $e, $pos, $series);
        } else {
            $vod_movie = $tfs->get_torrent_files_vod_movie(
                $stack, $autostart, $sel_id, $pos);
        }
        if (!$vod_movie)
            return null;

        MainHelper::$pb_stat->set_switch_reason('');

        hd_print("D: play(4) hash: ".$tfs->get_hash());

        $vod_info = $vod_movie->get_vod_info(
            $sel_id, -1, $tfs->get_switch_mode());
        return ActionFactory::vod_play($vod_info);
    }

    private function menu_playback_finish($user_input)
    {
        return TorrentHelper::menu_playback_finish($user_input,
            $this->custom_action('replace_path_after_playback'));
    }

    private function info($user_input)
    {
        $st = MainHelper::$state;
        $tfs = $st->nav_tfs;
        $mts = $st->nav_mts;
        $m = $mts->get_movie();
        $tor_series = $tfs->get_series();

        $sel = json_decode($user_input->selected_media_url);
        $sel_id = $sel ? HD::get($sel, 'id') : '';

        $sel_path = null;
        foreach ($tor_series as $path => $ser) {
            if ($ser['id'] == $sel_id) {
                $sel_path = $path;
                break;
            }
        }
        if (!$sel_path)
            return null;

        $ser = HD::get($tor_series, $sel_path);
        if (!$ser)
            return null;

        $dir = HD::path_dirname($sel_path);
        if ($dir == '')
            $name = $sel_path;
        else if ($dir == '/')
            $name = substr($sel_path, 1);
        else if (strlen($dir) + 1 < strlen($sel_path))
            $name = substr($sel_path, strlen($dir) + 1);
        else
            $name = $sel_path;

        $defs = array();

        ControlFactory::add_label($defs,
            'File:', $name, array('max_lines' => 2));
        ControlFactory::add_vgap($defs, -10);

        ControlFactory::add_label($defs,
            'Folder:', $dir, array('max_lines' => 5));

        if ($m->series_type) {
            $e = HD::get($ser, 'e', -1);
            $s = $e >= 0 ? HD::get($ser, 's', 1) : -1;
            if ($e >= 0)
                $str = T::g2('shell_ext_plugin_item_season_episode__2', $s, $e);
            else
                $str = 'N/A';

            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, 'S/E:', $str);
        }

        if ($this->debug) {
            ControlFactory::add_vgap($defs, -10);
            ControlFactory::add_label($defs, 'ID:', $ser['id']);
        }

        // TODO: some parsing details, if any?..

        ControlFactory::add_vgap($defs, 20);
        ControlFactory::add_close_dialog_button($defs,
            T::g('button_ok'), 250);

        return ActionFactory::show_dialog(
            T::g('navigator_popup_info'), $defs, true, 0);
    }

    private function popup($user_input)
    {
        if (!is_dir("/tmp/run/dl_input"))
            return null;
        if (!is_dir("/tmp/mnt/D/dune_downloads"))
            return null;

        $st = MainHelper::$state;
        $tfs = $st->nav_tfs;
        $torrent_id = $tfs->get_torrent_id();
        $group_id = $tfs->get_group_id();

        $sel = json_decode($user_input->selected_media_url);
        $sel_id = $sel ? HD::get($sel, 'id') : '';

        $dl = MainHelper::$dl_state;
        $dl->reload_state($group_id);

        $path = $tfs->find_series_by_id($sel_id);

        $f = HD::get($dl->cur_g->f_by_path, $path);

        $menu_items = array();
        if ($f)
        {
            $status = $f ? HD::get($f, 'status', 'N') : '';
            if ($status && $status != 'N')
            {
                $menu_items[] = ActionFactory::get_menu_item(
                    T::t('popup_action_show_in_folder'),
                    MainHelper::$entry_handler->custom_action(
                        'show_in_folder',
                        array('file_path' => $path)));
            }

            $params = array(
                'movie_id' => $tfs->get_movie_id(),
                'season' => $tfs->get_season(),
                'hash' => $tfs->get_hash());

            $menu_items[] = ActionFactory::get_menu_item(
                T::g('navigator_popup_delete'),
                MainHelper::$entry_handler->custom_action(
                    'delete_torrent_data',
                    array('torrent_id' => $torrent_id,
                        'file_path' => $path,
                        'params_str' => json_encode($params),
                    )));
        }
        else
        {
            $menu_items[] = ActionFactory::get_menu_item(
                T::t('popup_action_download'),
                MainHelper::$entry_handler->custom_action(
                    'download_torrent',
                    array('torrent_id' => $torrent_id,
                        'file_path' => $path)));
        }

        return ActionFactory::show_popup_menu($menu_items);
    }

    ///////////////////////////////////////////////////////////////////////

    public function set_fast_timer()
    {
        $this->fast_timer_countdown = 10;
    }

    public function speed_up_timer($parent_murl, &$plugin_cookies)
    {
        $this->set_fast_timer();
        return ActionFactory::change_behaviour(
            $this->get_action_map($parent_murl, $plugin_cookies),
            $this->get_timer($parent_murl));
    }

    private function timer($user_input, &$plugin_cookies)
    {
        $st = MainHelper::$state;
        $tfs = $st->nav_tfs;
        $group_id = $tfs->get_group_id();
        $dl = MainHelper::$dl_state;
        $changed = $dl->reload_state($group_id);

        $parent_murl = MediaURL::decode($user_input->parent_media_url);
        $a = ActionFactory::change_behaviour(
            $this->get_action_map($parent_murl, $plugin_cookies),
            $this->get_timer($parent_murl));
        if ($this->fast_timer_countdown > 0)
            $this->fast_timer_countdown--;

        if ($changed)
        {
            $a = ActionFactory::invalidate_folders(
                array($user_input->parent_media_url), $a);
        }

        return $a;
    }

    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        if (HD::get($user_input, 'control_id') == 'timer') {
            hd_print('torrent files D: timer');
        } else {
            hd_print('Torrent files screen: handle_user_input:');
            foreach ($user_input as $key => $value)
                hd_print("  $key => $value");
        }

        $control_id = HD::get($user_input, 'control_id');

        if ($control_id == 'timer')
            return $this->timer($user_input, $plugin_cookies);

        if ($control_id == 'info')
            return $this->info($user_input);

        if ($control_id == 'popup')
            return $this->popup($user_input);

        if ($control_id == 'enter')
            return $this->enter($user_input);

        if ($control_id == 'menu_playback_finish')
            return $this->menu_playback_finish($user_input);

        if ($control_id == 'replace_path_after_playback')
            return UrlHelper::replace_path_after_playback($user_input);

        return null;
    }
}
?>
